pub(crate) fn signature_to_ir(
signature: &FunctionType,
config: TargetFrontendConfig,
arch: Architecture,
) -> SignatureExpand description
Lower a WebAssembly signature to the native signature described by ReturnAbi.
pub(crate) fn signature_to_ir(
signature: &FunctionType,
config: TargetFrontendConfig,
arch: Architecture,
) -> SignatureLower a WebAssembly signature to the native signature described by ReturnAbi.