fn declare_locals(
builder: &mut FunctionBuilder<'_>,
count: u32,
wasm_type: ValType,
next_local: &mut usize,
environ: &mut FuncEnvironment<'_>,
) -> WasmResult<()>Expand description
Declare count local variables of the same type, starting from next_local.
Fail if the type is not valid for a local.