parse_function_body

Function parse_function_body 

fn parse_function_body(
    module_translation_state: &ModuleTranslationState,
    reader: &mut dyn FunctionBinaryReader<'_>,
    builder: &mut FunctionBuilder<'_>,
    state: &mut FuncTranslationState,
    environ: &mut FuncEnvironment<'_>,
    allow_unaligned_memory_accesses: bool,
) -> WasmResult<()>
Parse the function body in reader.

This assumes that the local variable declarations have already been parsed and function arguments and locals are declared in the builder.